
using namespace System.Management.Automation
using namespace System.Management.Automation.Language

Register-ArgumentCompleter -Native -CommandName 'rip' -ScriptBlock {
    param($wordToComplete, $commandAst, $cursorPosition)

    $commandElements = $commandAst.CommandElements
    $command = @(
        'rip'
        for ($i = 1; $i -lt $commandElements.Count; $i++) {
            $element = $commandElements[$i]
            if ($element -isnot [StringConstantExpressionAst] -or
                $element.StringConstantType -ne [StringConstantType]::BareWord -or
                $element.Value.StartsWith('-') -or
                $element.Value -eq $wordToComplete) {
                break
        }
        $element.Value
    }) -join ';'

    $completions = @(switch ($command) {
        'rip' {
            [CompletionResult]::new('--graveyard', 'graveyard', [CompletionResultType]::ParameterName, 'Directory where deleted files rest')
            [CompletionResult]::new('-d', 'd', [CompletionResultType]::ParameterName, 'Permanently deletes the graveyard')
            [CompletionResult]::new('--decompose', 'decompose', [CompletionResultType]::ParameterName, 'Permanently deletes the graveyard')
            [CompletionResult]::new('-s', 's', [CompletionResultType]::ParameterName, 'Prints files that were deleted in the current directory')
            [CompletionResult]::new('--seance', 'seance', [CompletionResultType]::ParameterName, 'Prints files that were deleted in the current directory')
            [CompletionResult]::new('-u', 'u', [CompletionResultType]::ParameterName, 'Restore the specified files or the last file if none are specified')
            [CompletionResult]::new('--unbury', 'unbury', [CompletionResultType]::ParameterName, 'Restore the specified files or the last file if none are specified')
            [CompletionResult]::new('-i', 'i', [CompletionResultType]::ParameterName, 'Print some info about TARGET before burying')
            [CompletionResult]::new('--inspect', 'inspect', [CompletionResultType]::ParameterName, 'Print some info about TARGET before burying')
            [CompletionResult]::new('-h', 'h', [CompletionResultType]::ParameterName, 'Print help')
            [CompletionResult]::new('--help', 'help', [CompletionResultType]::ParameterName, 'Print help')
            [CompletionResult]::new('-V', 'V ', [CompletionResultType]::ParameterName, 'Print version')
            [CompletionResult]::new('--version', 'version', [CompletionResultType]::ParameterName, 'Print version')
            [CompletionResult]::new('completions', 'completions', [CompletionResultType]::ParameterValue, 'Generate shell completions file')
            [CompletionResult]::new('graveyard', 'graveyard', [CompletionResultType]::ParameterValue, 'Print the graveyard path')
            [CompletionResult]::new('help', 'help', [CompletionResultType]::ParameterValue, 'Print this message or the help of the given subcommand(s)')
            break
        }
        'rip;completions' {
            [CompletionResult]::new('-h', 'h', [CompletionResultType]::ParameterName, 'Print help')
            [CompletionResult]::new('--help', 'help', [CompletionResultType]::ParameterName, 'Print help')
            break
        }
        'rip;graveyard' {
            [CompletionResult]::new('-s', 's', [CompletionResultType]::ParameterName, 'Get the graveyard subdirectory of the current directory')
            [CompletionResult]::new('--seance', 'seance', [CompletionResultType]::ParameterName, 'Get the graveyard subdirectory of the current directory')
            [CompletionResult]::new('-h', 'h', [CompletionResultType]::ParameterName, 'Print help')
            [CompletionResult]::new('--help', 'help', [CompletionResultType]::ParameterName, 'Print help')
            break
        }
        'rip;help' {
            [CompletionResult]::new('completions', 'completions', [CompletionResultType]::ParameterValue, 'Generate shell completions file')
            [CompletionResult]::new('graveyard', 'graveyard', [CompletionResultType]::ParameterValue, 'Print the graveyard path')
            [CompletionResult]::new('help', 'help', [CompletionResultType]::ParameterValue, 'Print this message or the help of the given subcommand(s)')
            break
        }
        'rip;help;completions' {
            break
        }
        'rip;help;graveyard' {
            break
        }
        'rip;help;help' {
            break
        }
    })

    $completions.Where{ $_.CompletionText -like "$wordToComplete*" } |
        Sort-Object -Property ListItemText
}
